@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 3
+ * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be
@@ -14,20 +14,20 @@
  * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  * PURPOSE. See the GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
+ * Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
+ *
  */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <string.h>
-
 #include "canvas.h"
 #include "callbacks.h"
-#include "util.h"
+#include "support.h"
 #include "debug.h"
 #include <gtk/gtk.h>
 
@@ -71,7 +71,8 @@ guint8 init_palette_values[][3] =
 
 const int NUM_PALETTES = sizeof(init_palette_values)/sizeof(*init_palette_values);
 const int RIGHT_BUTTON = 3;
-
+const guint8  MAX_8_BIT_COLOR  = 0xFF;
+const guint16 MAX_16_BIT_COLOR = 0xFFFF;
 
 typedef enum
 {
@@ -119,6 +120,16 @@ change_foreground_color(gpaint_canvas *canvas, GdkColor *color);
 
 static void
 change_background_color(gpaint_canvas *canvas, GdkColor *color);
+
+/*
+ * Convert an 8 bit color [0,255] to a 16 bit color [0,65538].
+ */
+static guint16
+color16(guint8 color)
+{
+    return ((guint16)color * (guint16)MAX_8_BIT_COLOR) + (guint16)color;
+}
+
 /*
  * Set the foreground color picker initial color.
  */
@@ -200,8 +211,7 @@ on_color_palette_entry_expose_event    (GtkWidget       *widget,
     gpaint_color_swatch *swatch = 
         (gpaint_color_swatch*)gtk_object_get_data(GTK_OBJECT(widget), "color_swatch");
     debug_fn1("widget=%s", gtk_widget_get_name(widget));
-    if (swatch == NULL)
-        return FALSE;
+    g_assert(swatch);
     gdk_draw_rectangle(
             widget->window, swatch->gc, TRUE,
             event->area.x, event->area.y,
