@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 3
+ * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be
@@ -16,18 +16,18 @@
  * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  * PURPOSE. See the GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
+ * Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA
  */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-
 #include "fill.h"
 #include "image.h"
-#include "util.h"
 #include "debug.h"
 
 #define STACKSIZE 10000
@@ -56,6 +56,11 @@ struct fillpixelinfo
 
 #define GPAINT_FILL(tool)  ((gpaint_fill*)(tool))
 
+#define SHIFT_COLOR(NAME) \
+ (color->pixel & visual->NAME##_mask) >> \
+ visual->NAME##_shift << \
+ (sizeof(unsigned char) * 8 - visual->NAME##_prec);
+
 #define PUSH(py, pxl, pxr, pdy) \
 { \
     struct fillpixelinfo *p = sp;\
@@ -81,10 +86,10 @@ struct fillpixelinfo
 
 static void fill_destroy(gpaint_tool *tool);
 static void fill_bounded_area(gpaint_tool *tool, int x, int y);
-
+static void convert_color(const GdkColor *color, unsigned char *r, unsigned char *g, unsigned char *b);
 static void flood_fill_algo(struct fillinfo *info, int x, int y);
-static void __inline__ set_new_pixel_value(struct fillinfo *info, int x, int y);
-static int __inline__ is_old_pixel_value(struct fillinfo *info, int x, int y);
+static void set_new_pixel_value(struct fillinfo *info, int x, int y);
+static int is_old_pixel_value(struct fillinfo *info, int x, int y);
 
 gpaint_tool *fill_create(const char *name)
 {
@@ -146,30 +151,33 @@ fill_bounded_area(gpaint_tool* tool, int x, int y)
     drawing_modified(drawing);
 }
 
-
-static __inline__ int
-is_old_pixel_value(struct fillinfo *info, int x, int y)
+static void
+convert_color(const GdkColor *color, unsigned char *r, unsigned char *g, unsigned char *b)
 {
-    unsigned char *p = info->rgb + y * info->rowstride + x * info->pixelsize;
-    unsigned char or, og, ob;
-    or = *p;
-    og = *(p + 1);
-    ob = *(p + 2);
-    if ((or == info->or) && (og == info->og) && (ob == info->ob))
+    GdkColormap *cmap = gdk_rgb_get_cmap();
+    GdkVisual *visual = gdk_rgb_get_visual();
+    GdkColor *c;
+    switch(visual->type)
     {
-        return 1;
+        case GDK_VISUAL_STATIC_COLOR:
+        case GDK_VISUAL_PSEUDO_COLOR:
+            c = cmap->colors + color->pixel;
+            *r = c->red >> 8;
+            *g = c->green >> 8;
+            *b = c->blue >> 8;
+            break;
+
+        case GDK_VISUAL_TRUE_COLOR:
+        case GDK_VISUAL_DIRECT_COLOR:
+            *r = SHIFT_COLOR(red);
+            *g = SHIFT_COLOR(green);
+            *b = SHIFT_COLOR(blue);
+            break;
+   
+       default:
+            g_assert_not_reached();
+            break;
     }
-    return 0;
-}
-
-
-static __inline__ void
-set_new_pixel_value(struct fillinfo *info, int x, int y)
-{
-    unsigned char *p = info->rgb + y * info->rowstride + x * info->pixelsize;
-    *p = info->r;
-    *(p + 1) = info->g;
-    *(p + 2) = info->b;
 }
 
 /*
@@ -232,3 +240,27 @@ skip:
 }  
 
 
+static int
+is_old_pixel_value(struct fillinfo *info, int x, int y)
+{
+    unsigned char *p = info->rgb + y * info->rowstride + x * info->pixelsize;
+    unsigned char or, og, ob;
+    or = *p;
+    og = *(p + 1);
+    ob = *(p + 2);
+    if ((or == info->or) && (og == info->og) && (ob == info->ob))
+    {
+        return 1;
+    }
+    return 0;
+}
+
+
+static void
+set_new_pixel_value(struct fillinfo *info, int x, int y)
+{
+    unsigned char *p = info->rgb + y * info->rowstride + x * info->pixelsize;
+    *p = info->r;
+    *(p + 1) = info->g;
+    *(p + 2) = info->b;
+}
